@@ -6,7 +6,6 @@ import requests
 from datetime import datetime, timezone
 import re
 import os
-import json
 
 from xml.etree import ElementTree
 # noinspection PyUnresolvedReferences
@@ -15,7 +14,9 @@ from xml.dom import minidom
 __all__ = ['EventPlanned', 'EventUnplanned', 'EventMedia', 'SignInventory', 'SignStatus',
            'RouteInventory', 'RouteStatus', 'RwisInventory', 'RwisStatus']
 
+#TODO make sql generator for cams and message signs
 
+# noinspection DuplicatedCode
 def _dict_keys_lower_case(d: dict):
     return_dict = dict()
     for k, v in d.items():
@@ -72,9 +73,16 @@ def _get_time_el(d: datetime, el_name: str):
 
 class _GeoJson:
 
+    @classmethod
+    @abc.abstractmethod
+    def get(cls, id_):
+        pass
+
     @classmethod
     def get_geojson(cls, id_: int = None):
         features: typing.List[_GeoJson] = cls.get(id_)
+        # cls.
+        # features: typing.List[_GeoJson] = cls.__dict__['get'](id_)
 
         return {
             "type": "FeatureCollection",
@@ -99,6 +107,11 @@ class _GeoJson:
             'properties': self.json
         }
 
+    @property
+    @abc.abstractmethod
+    def json(self):
+        pass
+
 
 class Atms:
     url: str = None
@@ -109,7 +122,7 @@ class Atms:
 
     @classmethod
     @abc.abstractmethod
-    def get(cls, id_: int = None):
+    def get(cls, id_: int = None) -> typing.List:
         if cls == Atms:
             raise NotImplementedError("Cannot call 'get' on base class")
 
@@ -180,8 +193,8 @@ class _Location:
         loc1_valid = loc1.latitude is not None and loc1.latitude != 0 and \
                      loc1.longitude is not None and loc1.longitude != 0
 
-        loc2_valid = loc2.latitude is not None and loc2.latitude != 0 and \
-                     loc2.longitude is not None and loc2.longitude != 0
+        loc2_valid = \
+            loc2.latitude is not None and loc2.latitude != 0 and loc2.longitude is not None and loc2.longitude != 0
 
         if loc1_valid and loc2_valid:
             if loc1.latitude == loc2.latitude and loc1.longitude == loc2.longitude:
@@ -357,7 +370,6 @@ class EventUnplanned(_EventBase):
         if accidents_and_incidents == 'crash':
             accidents_and_incidents = 'accident'
 
-
         event_headline = ElementTree.Element('event-headline')
         headline = ElementTree.Element('headline')
         headline.append(_el_tag_text('accidentsAndIncidents', accidents_and_incidents))
@@ -474,6 +486,15 @@ class EventPlanned(_EventBase):
 
         return ret
 
+    @property
+    def xml_el(self):
+        # print(self.id)
+        # print(json.dumps(self.json, indent=4))
+
+        el = ElementTree.Element('fullEventUpdate')
+
+        return el
+
 
 class EventMedia(Atms):
     _url_all = '/api/UnplannedEvents/EventMedia'
@@ -497,6 +518,10 @@ class EventMedia(Atms):
             'publishOn511': self._publish_on_511
         }
 
+    @property
+    def xml_el(self):
+        return super().xml_el
+
 
 class SignInventory(Atms, _GeoJson):
     _url_all = '/api/Signs'
@@ -530,7 +555,12 @@ class SignInventory(Atms, _GeoJson):
 
         return geo
 
+    @property
+    def xml_el(self):
+        return super().xml_el
 
+
+# noinspection DuplicatedCode
 class SignStatus(Atms):
     _url_all = '/api/Signs/Statuses'
     _url_by_id = '/api/Signs/{id}/Status'
@@ -644,6 +674,10 @@ class CameraInventory(Atms, _GeoJson):
 
         return geo
 
+    @property
+    def xml_el(self):
+        return super().xml_el
+
 
 class RouteInventory(Atms):
     _url_all = '/api/Routes'
@@ -659,7 +693,7 @@ class RouteInventory(Atms):
         self._free_flow_travel_time: int = data['freeflowtraveltime']
         self._start_location: _Location = _Location(data['startlocation'])
         self._end_location: _Location = _Location(data['endlocation'])
-        self._waypoints: typing.List[_Location] = [_Location(l) for l in data['waypoints']]
+        self._waypoints: typing.List[_Location] = [_Location(ll) for ll in data['waypoints']]
         super().__init__(data)
 
     @property
@@ -682,8 +716,13 @@ class RouteInventory(Atms):
     def route_length(self):
         return self._routeLength
 
+    @property
+    def xml_el(self):
+        return super().xml_el
+
 
 class RouteStatus(Atms):
+
     _url_all = '/api/Routes/Statuses'
     _url_by_id = '/api/Routes/{id}/Status'
 
@@ -732,7 +771,7 @@ class RouteStatus(Atms):
         el.append(_el_tag_text('route-status', '1'))
 
         if self._rte_len > -1:
-            el.append(_el_tag_text('total-distance', int(self._rte_len * 1600)))
+            el.append(_el_tag_text('total-distance', int(self._rte_len)))
 
         el.append(_el_tag_text('display-travel-time', self._displayed_tt))
         el.append(_el_tag_text('calculated-travel-time', self._calculated_tt))
@@ -776,6 +815,10 @@ class RwisInventory(Atms):
 
         return ret
 
+    @property
+    def xml_el(self):
+        return super().xml_el
+
 
 class RwisStatus(Atms):
     _url_all = '/api/RWIS/Statuses'
@@ -798,6 +841,10 @@ class RwisStatus(Atms):
 
         return ret
 
+    @property
+    def xml_el(self):
+        return super().xml_el
+
 
 def printd(*args):
     """
